"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Node = /** @class */ (function () {
    function Node(key) {
        this.key = key;
        this.neighbours = [];
    }
    Node.prototype.addNeighbour = function (node) {
        this.neighbours.push(node);
    };
    return Node;
}());
var Graph = /** @class */ (function () {
    function Graph(_a) {
        var _b = (_a === void 0 ? {} : _a).directed, directed = _b === void 0 ? false : _b;
        this.nodes = [];
        this.edges = new Map();
        this.directed = directed;
    }
    Graph.prototype.addNode = function (key) {
        this.nodes.push(new Node(key));
    };
    Graph.prototype.getNode = function (key) {
        var node = this.nodes.find(function (node) { return node.key === key; });
        if (!node) {
            throw new Error("Could not find node " + key);
        }
        return node;
    };
    Graph.prototype.addEdge = function (key1, key2, weight) {
        var startNode = this.getNode(key1);
        var endNode = this.getNode(key2);
        if (this.directed) {
            this.edges.set(key1 + "-" + key2, weight);
            startNode.addNeighbour(endNode);
        }
        else {
            var _a = [key1, key2].sort(function (a, b) { return a.localeCompare(b); }), sort1 = _a[0], sort2 = _a[1];
            this.edges.set(sort1 + "-" + sort2, weight);
            startNode.addNeighbour(endNode);
            endNode.addNeighbour(startNode);
        }
    };
    Graph.prototype.getEdge = function (key1, key2) {
        if (this.directed) {
            return this.edges.get(key1 + "-" + key2);
        }
        else {
            var _a = [key1, key2].sort(function (a, b) { return a.localeCompare(b); }), sort1 = _a[0], sort2 = _a[1];
            return this.edges.get(sort1 + "-" + sort2);
        }
    };
    Graph.prototype.findLowest = function (dist, visited) {
        return Object.entries(dist).reduce(function (acc, val) {
            return val[1].distance < acc[1].distance && !visited.has(val[0]) ? val : acc;
        }, ['', { distance: Infinity, previous: null }]);
    };
    Graph.prototype.dijkstra = function (source) {
        var _this = this;
        var results = {};
        var visited = new Set();
        this.nodes.forEach(function (_a) {
            var key = _a.key;
            results[key] = {
                distance: Infinity,
                previous: null,
            };
        });
        results[source] = {
            distance: 0,
            previous: null,
        };
        var _loop_1 = function () {
            var lowestKey = this_1.findLowest(results, visited)[0];
            visited.add(lowestKey);
            var node = this_1.getNode(lowestKey);
            node.neighbours.forEach(function (neighbour) {
                if (visited.has(neighbour.key))
                    return;
                var edgeWeight = _this.getEdge(node.key, neighbour.key);
                var alt = results[node.key].distance + (edgeWeight || Infinity);
                if (alt < results[neighbour.key].distance) {
                    results[neighbour.key] = {
                        distance: alt,
                        previous: node.key,
                    };
                }
            });
        };
        var this_1 = this;
        while (visited.size < this.nodes.length) {
            _loop_1();
        }
        return results;
    };
    Graph.prototype.findPath = function (source, destination) {
        if (!this.nodes.find(function (_a) {
            var key = _a.key;
            return key === destination;
        })) {
            throw new Error("Could not find node " + destination);
        }
        var results = this.dijkstra(source);
        function buildRoute(key, route) {
            if (route === void 0) { route = []; }
            var previous = results[key].previous;
            return previous === null
                ? [key].concat(route) : buildRoute(previous, [key].concat(route));
        }
        return buildRoute(destination);
    };
    Graph.prototype.bfs = function (startKey, fn) {
        var visited = new Set();
        var q = [];
        q.unshift(this.getNode(startKey));
        while (q.length > 0) {
            var currentNode = q.pop();
            if (!visited.has(currentNode.key)) {
                fn(currentNode);
                visited.add(currentNode.key);
            }
            currentNode.neighbours.forEach(function (node) {
                if (!visited.has(node.key)) {
                    q.unshift(node);
                }
            });
        }
    };
    Graph.prototype.dfs = function (startKey, fn) {
        var visited = new Set();
        function explore(node) {
            if (visited.has(node.key))
                return;
            fn(node);
            visited.add(node.key);
            node.neighbours.forEach(function (node) {
                explore(node);
            });
        }
        explore(this.getNode(startKey));
    };
    return Graph;
}());
exports.Graph = Graph;
